/*
 * flash.c
 *
 *  Created on: Jun 23, 2023
 *      Author: true
 */

#include "hk32f030m.h"
#include "flash.h"



/**
 * @brief   This function erases the user application area, from start to end of flash.
 * @return  status: Report about the success of the erasing.
 */
__attribute__ ((long_call, section(".ramfunc"))) BL_flash_status flash_erase_user_app()
{
	// uint32_t timeout;
	uint32_t page;

	FLASH_Unlock();

	for (page = USER_APP_START_ADDR; page < USER_APP_END_ADDR; page += FLASH_ERASE_PAGE_SIZE) {
		if (FLASH_ErasePage(page) != FLASH_COMPLETE) {
			FLASH_Lock();
			return FLASH_ERROR;
		}

		/*
		// clear end of operation bit
		FLASH->SR &= ~FLASH_FLAG_EOP;

		// start erasing page
	    FLASH->CR |= FLASH_CR_PER;
	    FLASH->AR  = page;
	    FLASH->CR |= FLASH_CR_STRT;

	    // wait wait for operation to complete
		timeout = FLASH_ER_PRG_TIMEOUT;
		while ((FLASH->SR & FLASH_FLAG_BSY)) {
			timeout--;
			if (!timeout) {
				FLASH_Lock();
				return FLASH_ERROR;
			}
		}

		// disable page erase bit
		FLASH->CR &= ~FLASH_CR_PER;
		*/
	}

	FLASH_Lock();

	return FLASH_OK;
}

/**
 * @brief   This function flashes the memory.
 * @param   addr:    First address to be written to.
 * @param   *data:   Array of the data that we want to write.
 * @param   *length: Size of the data array, in 16-bit words.
 * @return  status:  Report about the success of the writing.
 */
__attribute__ ((long_call, section(".ramfunc"))) BL_flash_status flash_write(uint32_t addr, uint8_t *data, uint16_t length)
{
	BL_flash_status status = FLASH_OK;

	FLASH_Unlock();

	/* Loop through the data. */
	for (uint32_t i = 0u; (i < length) && (status == FLASH_OK); i++) {
	  if (addr >= USER_APP_END_ADDR) {
		  status = FLASH_ERROR_SIZE;
		  break;
	  } else {
		  /* The actual flashing. If there is an error, then report it. */
		  if (FLASH_ProgramByte(addr, data[i]) != FLASH_COMPLETE) {
			  status |= FLASH_ERROR_WRITE;
		  }

		  /* Read back the content of the memory. If it is wrong, then report an error. */
		  if (data[i] != (*(volatile uint8_t *)addr)) {
			  status |= FLASH_ERROR_VERIFY;
		  }

		  /* Shift the address by a word. */
		  addr++;
	  }
	}

	FLASH_Lock();

	return status;
}

void jump_to_user_app()
{
	void (*user_app)(void);

	// stop systick interrupt; reset to defaults
	SysTick->CTRL = 0;
	SysTick->LOAD = 0xffffff;
	SysTick->VAL  = 0xffffff;

	// stop USART; reset to defaults
	USART1->CR1 = 0;
	USART1->CR1 = 0;
	USART1->BRR = 0;

	// disable configured interrupts
	// NVIC->ICER[0] = 0;

	// configure jump location
	user_app = (void (*)(void))(*(volatile uint32_t *)(USER_APP_START_ADDR + 4));

	// configure stack pointer
	__set_MSP(*(volatile uint32_t*)USER_APP_START_ADDR);

	// configure interrupt vector table
	// SCB->VTOR = USER_APP_START_ADDR;

	// cortex m0 doesn't have VTOR to remap vector table.
	// to address this, HK MCU has a flash offset thing which
	// remaps 256 bytes specified by an offset, to to 0x08000000
	FLASH->INT_VEC_OFFSET = USER_APP_START_ADDR & 0x3ffc;

	// let's do this
	user_app();
}
